/*=======================================================================
  $Id: ind_choose.c,v 1.15 2007/04/25 17:20:57 andrew Exp $
 * All files in the distribution of the DPS system are Copyright
 * 1996 by the Computational Biology group in the Department of Biological
 * Sciences at Purdue University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all such
 * copies, and that any documentation, announcements, and other materials
 * related to such distribution and use acknowledge that the software was
 * developed by the Computational Biology group in the Department of
 * Biological Sciences at Purdue University, W. Lafayette, IN by Ingo
 * Steller and Michael G. Rossmann. No charge may be made for copies,
 * derivations, or distributions of this material without the express
 * written consent of the copyright holder.  Neither the name of the
 * University nor the names of the authors may be used to endorse or
 * promote products derived from this material without specific prior
 * written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 *======================================================================*/

/*=====================================================================*
 *                                                                     *
 *                         Data Processing Suit                        *
 *                                                                     *
 *                              dps_index                              *
 *                                                                     *
 *                        Written by Ingo Steller                      *
 *                                                                     *
 *                          File: ind_choose.c                         *
 *                                                                     *
 *=====================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "index.h"
#include "vector_math.h"
#include "ind_choose.h"
#include "util.h"
/* choose_non_lin(): Choose the first three non linear depended 	*
 *		direct lattice vectors.					*/
int choose_non_lin(	int dumpch,float sinthmin, float volmin, float hkldev, float hklmin,
                        int number_xyz, struct vector xyz[],
			int t_vectors, struct tvector t[], int max_om,
			struct matrix_3x3 *om )
{
  int i,j,k,imax,jmax,kmax;		/* counter */
  float	volume;		/* volume of the cell */
  float	tmp;		/* temporary variable */
  int 	om_count = 0;   /* Found orientation matrices */
  int a,b,c;		/* Number of vectors for generating om after
			   sorting */
  float vec_scale[N_TVECTOR],sin_vector[4],sin_vector_max,dtor,rtod;
  struct vector newvec[N_TVECTOR];
  int tmp_indexed,max_indexed;
  struct hkl_int	*hkl;		/* Integer hkl, same index as xyz */
  struct hkl_fract *hkl_calc;	/* Fractional hkl, same index as xyz */
  struct matrix_3x3 junk;
  FILE *chooseout;
  hkl = dps_malloc("choose_best_om", "hkl", sizeof(struct hkl_int)*number_xyz);
  hkl_calc = dps_malloc("choose_best_om", "hkl_calc", sizeof(struct hkl_fract)*number_xyz);

  /* hrp 04-03-2000 start This finds the solution which has 
     the largest product of the sines of the three cell angles, 
     to allow a cutoff to around half this value to reject 
     really bad solutions. Also the maximum number of reflections
     indexed by any solution is found, and the really bad ones 
     rejected at this stage.
  */

  max_indexed = 0;
  sin_vector_max = 0.0;
  if (dumpch != 0) 
      printf("\n In choose_non_lin, sinthmin %f volmin %f hkldev %f hklin %f \n",
	     sinthmin,  volmin,  hkldev,  hklmin);

  for(i=0;i<=t_vectors;i++){
    vec_scale[i] = 1.0/vec_lenf(t[i].vec);
    newvec[i].x = vec_scale[i]*t[i].vec.x;
    newvec[i].y = vec_scale[i]*t[i].vec.y;
    newvec[i].z = vec_scale[i]*t[i].vec.z;
  }
  for(i=2;i<=t_vectors;i++) {
    for(j=0;j<=i-2;j++) {
      for(k=j+1;k<=i-1;k++) {

	sin_vector[0] = sin(acos((newvec[i].x*newvec[j].x)+
				 (newvec[i].y*newvec[j].y)+
				 (newvec[i].z*newvec[j].z)));

	sin_vector[1] = sin(acos((newvec[i].x*newvec[k].x)+
				 (newvec[i].y*newvec[k].y)+
				 (newvec[i].z*newvec[k].z)));

	sin_vector[2] = sin(acos((newvec[k].x*newvec[j].x)+
				 (newvec[k].y*newvec[j].y)+
				 (newvec[k].z*newvec[j].z)));

	sin_vector[3] = fabs(sin_vector[0]*sin_vector[1]*
			     sin_vector[2]);
	if (sin_vector[3]>sin_vector_max){
	  sin_vector_max = sin_vector[3];
	}
	om_from_vectors(t[i].vec,t[j].vec,t[k].vec,&om[0]);

	/* Generate fractional and integer (rounded) HKL */
	xyz_2_hkl(om[0], xyz, number_xyz, hkl, hkl_calc);
	/* Calculate the number of reflections indexable with this OM. */
	/*	tmp_indexed = num_ok_hkl(hkl_calc, number_xyz, hkl, MAX_DIFF_HKL); */
	tmp_indexed = num_ok_hkl(hkl_calc, number_xyz, hkl, hkldev);
  if (dumpch != 0) 
	  printf(" number spots indexed %d for vectors %d %d %d triplesin %f\n", 
                tmp_indexed,i,j,k,sin_vector[3]);

	if(max_indexed<tmp_indexed){
	  max_indexed=tmp_indexed;
          imax=i;
          jmax=j;
          kmax=k;
	}
      }
    }
  }
  /* Debug output */
  if (dumpch != 0) 
	  printf("max value of triple sine product %f  max number spots indexed %d for vectors %d %d %d\n", 
                sin_vector_max,max_indexed,imax,jmax,kmax);

  max_indexed = max_indexed*hklmin;

  /* hrp 04-03-2000 end insert */
  if (dumpch != 0) 
	  printf("\n max_indexed now set to %d for tests\n",max_indexed);
  if (dumpch != 0) 
      printf("\n now testing triple sin and volume\n");

  for(i=2;i<=t_vectors;i++) {
    for(j=0;j<=i-2;j++) {
      for(k=j+1;k<=i-1;k++) {

	sin_vector[0] = sin(acos((newvec[i].x*newvec[j].x)+
				 (newvec[i].y*newvec[j].y)+
				 (newvec[i].z*newvec[j].z)));

	sin_vector[1] = sin(acos((newvec[i].x*newvec[k].x)+
				 (newvec[i].y*newvec[k].y)+
				 (newvec[i].z*newvec[k].z)));

	sin_vector[2] = sin(acos((newvec[k].x*newvec[j].x)+
				 (newvec[k].y*newvec[j].y)+
				 (newvec[k].z*newvec[j].z)));

	sin_vector[3] = fabs(sin_vector[0]*sin_vector[1]*
			     sin_vector[2]);
        if (dumpch != 0) printf("i,j,k %d %d %d triplesin %f \n",i,j,k,sin_vector[3]);
	if(sin_vector[3]>sinthmin*sin_vector_max){ 
	  volume = vec_volumef(t[i].vec,t[j].vec,t[k].vec);
	  tmp=t[i].len * t[j].len * t[k].len;
          if (dumpch != 0) printf("  volume %f tmp %f \n",volume,tmp);
	  /*	  if ( (volume > tmp*LIN_DEP_FACTOR) ) { */
	  if ( (volume > tmp*volmin) ) {
	    if (t[i].len > t[j].len) {
	      if (t[i].len > t[k].len) {
		a=i;
		b = t[j].len >= t[k].len ? j : k;
		c = t[j].len > t[k].len ? k : j;
	      }
	      else {
		a=k;
		b=i;
		c=j;
	      }
	    }
	    else {
	      if (t[j].len > t[k].len) {
		a=j;
		b = t[i].len >= t[k].len ? i : k;
		c = t[i].len > t[k].len ? k : i;
	      }
	      else {
		a=k;
		b=j;
		c=i;
	      }
	    }

	    om_from_vectors(t[c].vec,t[b].vec,t[a].vec,&om[om_count]);
	    /* Generate fractional and integer (rounded) HKL */
	    xyz_2_hkl(om[om_count], xyz, number_xyz, hkl, hkl_calc);
	    /* Calculate the number of reflections indexable with this OM. */
	    /*	    tmp_indexed = num_ok_hkl(hkl_calc, number_xyz, hkl, MAX_DIFF_HKL); */
	    tmp_indexed = num_ok_hkl(hkl_calc, number_xyz, hkl, hkldev);

	    if (dumpch != 0) 
	      printf("number %d chosen %d %d %d vol %f tmp %f rel %f indexed %d max %d\n", 
              om_count,c,b,a,volume,tmp,tmp/volume,tmp_indexed,max_indexed);      

	    if(tmp_indexed >= max_indexed){
	      ++om_count;
	      if (dumpch != 0) printf("stored this matrix\n");
	      if (om_count > (max_om-1)) return(om_count-1);
	    }
	  }
	  /*	  else {
            if (dumpch != 0) 
		printf("vectors i,j,k %d %d %d rejected, on volume: actual %f minimum %f\n",
                        i,j,k,volume,tmp*volmin);
			}*/

	}
	/*else {
          if (dumpch != 0) 
	  printf("vectors i,j,k %d %d %d rejected, triplesine only %f \n", 
		 i,j,k,sin_vector[3]);
		 }*/
      }
    }
  }
  if (dumpch != 0) printf("CHOOSE_NON_LINEAR left with %d matrices ...\n",om_count);
  return(om_count);

}

/* choose_best_om(): Given a list of possible orientation matrices, this routine returns the	*
 *		     number of the 'best' OM. This is decided on the number of reflections	*
 *		     that can be indexed within a certain margin around inter (MAX_DIFF_HKL)	*
 *		     and on the size of the unit cell. In principle the smallest unit cell	*
 *		     with the highest number of reflections is choosen.				*
 * Prerequirements: The list of orientation matrices should start with a OM generated by the	*
 *		    three shortest vectors (See routine index() ).				*/

int choose_best_om(int dumpch, float hkldev, int num_om, struct matrix_3x3 om[], int number_xyz, struct vector xyz[])

{
  int i,dpserrflg;			/* Counter */
  int best_indexed;	/* Number of indexed reflections for the currently chosen OM */
  int tmp_indexed;	/* Temporarely variable, same usage as best_indexed. */
  float best_volume;	/* Volume of the currently choosen OM */
  float tmp_volume;	/* Temporary variable, same usage as best_volume. */
  struct hkl_int	*hkl;		/* Integer hkl, same index as xyz */
  struct hkl_fract *hkl_calc;	/* Fractional hkl, same index as xyz */
  int selected;		/* Finally selected best OM. */
  struct matrix_3x3 junk;

  if (dumpch != 0) 
      printf("\n Choose_best_om entered, number of matrices: %d\n",num_om);

  /* Allocate memory for the hkl arrays. */
  hkl = dps_malloc("choose_best_om", "hkl", sizeof(struct hkl_int)*number_xyz);
  hkl_calc = dps_malloc("choose_best_om", "hkl_calc", sizeof(struct hkl_fract)*number_xyz);

  /* Set starting values, so that we definitely reach the statements of the 'if' statement */
  best_indexed = 0;
  best_volume = 1.0/vec_volumef(om[0].r[0], om[0].r[1], om[0].r[2]);

  /* Go through all OM's and find the 'best' */

  for(i=0;i<num_om;i++) {
    /* Generate fractional and integer (rounded) HKL */
    xyz_2_hkl(om[i], xyz, number_xyz, hkl, hkl_calc);
    /* Calculate the number of reflections indexable with this OM. */
    /*    tmp_indexed = num_ok_hkl(hkl_calc, number_xyz, hkl, MAX_DIFF_HKL); */
    tmp_indexed = num_ok_hkl(hkl_calc, number_xyz, hkl, hkldev);
    /* Calculate the volume of this OM. */
    tmp_volume = 1.0/vec_volumef(om[i].r[0], om[i].r[1], om[i].r[2]);
    /* Decide whether this OM is 'better' than the best found until now */
    /* zeroth: has it actually indexed any reflections? */
    if (tmp_indexed<1) {
      dpserrflg = 1;
      /* er - this was return(1) which failed if there was no solution better than 0! 
	 Harry 02-10-2002 */
      return(0);
    }
    /* First: Is the number of indexable reflections higher? */
    if (dumpch != 0) {
    printf("OM %d: indexed: %d volume %f\n",i,tmp_indexed,tmp_volume);
    print_3x3(om[i]);
    junk = om[i];
    print_cell_from_om(junk);
    xyz_2_hkl(junk, xyz, number_xyz, hkl, hkl_calc);
    /*    tmp_indexed = num_ok_hkl(hkl_calc, number_xyz, hkl, MAX_DIFF_HKL); */
    tmp_indexed = num_ok_hkl(hkl_calc, number_xyz, hkl, hkldev);
    printf("Number of indexed reflections %d\n\n",tmp_indexed);
    }

    if (best_indexed < tmp_indexed) {
      /* If yes, is the volume not too much higher. This should only avoid	    *
       * finding a higher order cell ( face/body centered etc.) which would have  *
       * twice the volume							    */
      /*hrp 01-03-2000*/

      if (best_volume*VOLUME_FACTOR > tmp_volume) {

	/* Ok, OM is better... */
	best_indexed = tmp_indexed;
	selected = i;
	best_volume=tmp_volume;

      }
    }
  }
  if (dumpch != 0) printf("CHOOSE_BEST_OM left with %d selected...\n",selected);

  return(selected);
}
