#!/bin/csh -f
# if there's no gfortran here, next line will not include "gcc version"
# and the three GFORTRAN_M..OR_*VERSION envs won't be set, so it should 
# be okay.
# 
# Once I'm happy with this I'll add the Intel compilers as well.
#
if ( $?CLIB == 0 ) then
    echo You must have CCP4 set up first! exiting now
endif
#
# needed for some mangled locales...
# 
setenv LC_ALL "C"
setenv BREAKOUTNOW 0
set gfortran_version = `gfortran -v > & junk ; grep 'gcc version' junk | awk '{print $3}' ; /bin/rm -f junk`
set GFORTRAN_MAJOR_VERSION = `echo $gfortran_version | cut -c1`
set GFORTRAN_MINOR_VERSION = `echo $gfortran_version | cut -c3`
set GFORTRAN_MINOR_SUBVERSION = `echo $gfortran_version | cut -c5`
setenv USE_G77 1
setenv USE_GFORTRAN 1
if ( $GFORTRAN_MINOR_VERSION < 1 ) then 
    echo your gfortran is too old, trying g77
else
#
# test for CCP4 libraries built as 32-bit objects with gfortran
#
    if ( $GFORTRAN_MINOR_VERSION < 2 ) then
    echo Your installed version of gfortran is not reliable - you should update to
    echo a version no older than 4.2.0, and preferably the newset stable version.
    endif
    echo testing gfortran -m32
    gfortran -m32 -c test.f
    if ( $status != 0 ) then
	echo
	echo You seem to have gfortran installed, but it cannot compile files
	echo properly. Check to see that you have set "LD_LIBRARY_PATH", and that
	echo there is a file called "libmpfr.so.1" \(or similar\) in that location
	setenv BREAKOUTNOW 1
#	exit
    endif
    gfortran -m32 test.o $CLIB/libccp4f.a $CLIB/libccp4c.a -o test_exec >& /dev/null
#
# need to make sure the executable both exists and runs
#
    set USE_GFORTRAN = $status
    set USE_GFORTRAN_32 = $USE_GFORTRAN
    if ( -e test_exec ) then
# this construction prevents "Segmentation fault" appearing on screen
	( ./test_exec && true ) >& /dev/null
		set USE_GFORTRAN_32 = $status
    else
	set USE_GFORTRAN_32 = 1
    endif
    echo USE_GFORTRAN is now $USE_GFORTRAN 
    if ( $USE_GFORTRAN == 0 ) then
	if ( $USE_GFORTRAN_32 == 0 ) then 
	    setenv BYTES32OR64 "-m32"
	    unsetenv BREAKOUTNOW
	    echo ... looks like a gfortran 32-bit build
	endif
    else
#
# or are the CCP4 libraries 64-bit?
#
        echo testing for 64-bit libraries built with gfortran
        gfortran -m64 -c test.f 
	if ( $status != 0 ) then
	    echo
	    echo You seem to have gfortran installed, but it cannot compile files
	    echo with the \"-m64\" flag.
	    set USE_GFORTRAN_64 = 1
	    unsetenv BREAKOUTNOW
    #	exit
	else
	    gfortran -m64 test.o $CLIB/libccp4f.a $CLIB/libccp4c.a -o test_exec >& /dev/null
	    set USE_GFORTRAN = $status
	    set USE_GFORTRAN_64 = $USE_GFORTRAN
	    if ( -e test_exec ) then 
		( ./test_exec && true ) >& /dev/null
		set USE_GFORTRAN_64 = $status
	    else
		set USE_GFORTRAN_64 = 1
	    endif
	endif
	
	if ( $USE_GFORTRAN_64 != 0 ) then 
	    echo ... your CCP4 libraries seem to have been built with gfortran,
	    echo     but not in either 32- or 64- bit mode. Sorry, this build script 
	    echo     will not work in this case! Contact harry@mrc-lmb.cam.ac.uk,
	    echo     including this entire message, for advice.
	    exit
	else
	    unsetenv BREAKOUTNOW
	    setenv BYTES32OR64 ""
	    echo ... looks like a gfortran 64-bit build
	endif
    else
	echo we can\'t use gfortran, sorry, because Your CCP4 libraries seem to be
	echo incompatible with your gfortran compiler. Let\'s try g77 instead
    endif
endif
if ( $USE_GFORTRAN == 0 ) then
    echo Your gfortran works with your CCP4 libraries, so we are building with 
    echo gfortran version $gfortran_version
    echo `which gfortran`
    setenv F77 "`which gfortran` ${BYTES32OR64}"
    setenv FC "${F77}"
    setenv CC "${F77}"
    setenv FLINK "${F77}"
else
    set g77_version = `g77 -v > & junk ; grep 'gcc version' junk | awk '{print $3}' ; /bin/rm -f junk`
#
# just need to make sure this is set, don't really have to worry about 
# which version it is.
# should also check to make sure that the CCP4 libs are compatible...
# 
# need to test 32 or 64-bit compiler.
    g77 -m32 -c test.f >& /dev/null
#
# some g77s (even 3.4, where it is normally found) don't have the "-m32" flag.
#
    set NO_M32_FLAG = $status
    if ( $NO_M32_FLAG != 0 ) then
	echo this version of g77 doesn\'t have the "-m32" flag
	g77 -c test.f
	setenv NO32BITFLAG 
    else
	g77 -m32 -c test.f
    endif
    g77 test.o $CLIB/libccp4f.a $CLIB/libccp4c.a >& /dev/null
    set USE_G77 = $status
    if ( $USE_G77 == 0 ) then 
	if ( $?NO32BITFLAG == 0 ) then
	setenv BYTES32OR64 "-m32"
	else
	setenv BYTES32OR64 ""
	echo no 32 bit flag for this compiler
	endif
	unsetenv BREAKOUTNOW
	echo ... looks like a g77 32-bit build
    else
	g77 -m64 -c test.f  
	if ( $status != 0 ) then
	    echo
	    echo You seem to have g77 installed, but it cannot compile files
	    echo with the \"-m64\" flag, so we will proceed with no \"-m\" flag.
	    set BYTES32OR64 ""
    #	exit
	else
	    g77 -m64 test.o $CLIB/libccp4f.a $CLIB/libccp4c.a >& /dev/null
	    set USE_G77 = $status
	    if ( $USE_G77 == 0 ) then 
		setenv BYTES32OR64 ""
		unsetenv BREAKOUTNOW
		echo ... looks like a g77 64-bit build
	    endif
	endif
    endif
    if ( $USE_G77 != 0 ) then
	echo we can\'t use g77, because your CCP4 libraries seem to be 
	echo incompatible with g77.
	echo Also, your gfortran is too old to produce a reliable executable 
        echo of Mosflm. You should either install a newer version of gfortran 
	echo \(version 4.2.0 is the earliest we recommend\), or download a 
        echo pre-built executable.
    else
	echo You don\'t seem to have a suitable gfortran, but g77 works with 
        echo your CCP4 libraries. Building with g77 version $g77_version
	unsetenv BREAKOUTNOW
	setenv F77 "`which g77` -fno-globals ${BYTES32OR64}"
	setenv FC "${F77}"
	setenv FCOMP "${F77}"
	setenv CC "`which gcc` ${BYTES32OR64}"
	setenv CCOMP "`which gcc` ${BYTES32OR64}"
	setenv FLINK "${F77} -lg2c"
    endif
endif
if ( -e a.out ) then
    /bin/rm -f a.out
endif

